@@ -610,7 +610,7 @@ static void
 init_closure_free (gpointer data)
 {
 	InitClosure *closure = data;
-	g_slice_free (InitClosure, closure);
+	g_free (closure);
 }
 
 static gboolean
@@ -717,18 +717,41 @@ secret_service_initable_iface (GInitableIface *iface)
 	iface->init = secret_service_initable_init;
 }
 
+static void
+secret_service_async_initable_init_async (GAsyncInitable *initable,
+                                          int io_priority,
+                                          GCancellable *cancellable,
+                                          GAsyncReadyCallback callback,
+                                          gpointer user_data);
+
+typedef struct {
+	GAsyncReadyCallback callback;
+	gpointer user_data;
+} InitBaseClosure;
+
 static void
 on_init_base (GObject *source,
               GAsyncResult *result,
               gpointer user_data)
 {
-	GTask *task = G_TASK (user_data);
+	GTask *base_task = G_TASK (user_data);
+	InitBaseClosure *base = g_task_get_task_data (base_task);
+	GCancellable *cancellable = g_task_get_cancellable (base_task);
+	GTask *task;
+	InitClosure *init;
 	SecretService *self = SECRET_SERVICE (source);
 	GError *error = NULL;
 
+	task = g_task_new (source, cancellable, base->callback, base->user_data);
+	g_task_set_source_tag (task, secret_service_async_initable_init_async);
+	init = g_new0 (InitClosure, 1);
+	g_task_set_task_data (task, init, init_closure_free);
+
+	g_clear_object (&base_task);
+
 	if (!secret_service_async_initable_parent_iface->init_finish (G_ASYNC_INITABLE (self),
 	                                                              result, &error)) {
-		g_task_return_error (task, error);
+		g_task_return_error (task, g_steal_pointer (&error));
 	} else {
 		service_ensure_for_flags_async (self, self->pv->init_flags, task);
 	}
@@ -744,12 +767,14 @@ secret_service_async_initable_init_async (GAsyncInitable *initable,
                                           gpointer user_data)
 {
 	GTask *task;
-	InitClosure *closure;
+	InitBaseClosure *base;
 
-	task = g_task_new (initable, cancellable, callback, user_data);
+	task = g_task_new (initable, cancellable, NULL, NULL);
 	g_task_set_source_tag (task, secret_service_async_initable_init_async);
-	closure = g_slice_new0 (InitClosure);
-	g_task_set_task_data (task, closure, init_closure_free);
+	base = g_new0 (InitBaseClosure, 1);
+	base->callback = callback;
+	base->user_data = user_data;
+	g_task_set_task_data (task, base, g_free);
 
 	secret_service_async_initable_parent_iface->init_async (initable,
 	                                                        io_priority,
@@ -797,7 +822,7 @@ secret_service_real_ensure_for_flags (SecretBackend *self,
 	g_return_if_fail (SECRET_IS_SERVICE (self));
 
 	task = g_task_new (self, cancellable, callback, user_data);
-	closure = g_slice_new0 (InitClosure);
+	closure = g_new0 (InitClosure, 1);
 	g_task_set_task_data (task, closure, init_closure_free);
 	service_ensure_for_flags_async (SECRET_SERVICE (self), flags, task);
 	g_object_unref (task);
@@ -979,7 +1004,7 @@ secret_service_get (SecretServiceFlags flags,
 	} else {
 		task = g_task_new (service, cancellable, callback, user_data);
 		g_task_set_source_tag (task, secret_service_get);
-		closure = g_slice_new0 (InitClosure);
+		closure = g_new0 (InitClosure, 1);
 		closure->flags = flags;
 		g_task_set_task_data (task, closure, init_closure_free);
 
@@ -1595,7 +1620,7 @@ ensure_closure_free (gpointer data)
 {
 	EnsureClosure *closure = data;
 	g_hash_table_unref (closure->collections);
-	g_slice_free (EnsureClosure, closure);
+	g_free (closure);
 }
 
 static void
@@ -1670,7 +1695,7 @@ secret_service_load_collections (SecretService *self,
 
 	task = g_task_new (self, cancellable, callback, user_data);
 	g_task_set_source_tag (task, secret_service_load_collections);
-	closure = g_slice_new0 (EnsureClosure);
+	closure = g_new0 (EnsureClosure, 1);
 	closure->collections = collections_table_new ();
 	g_task_set_task_data (task, closure, ensure_closure_free);
 
